/*
  dwOpenClient.h - MQTT deviceWISE Open Client.
*/

#ifndef dwOpenClient_h
#define dwOpenClient_h

#include <Process.h>
#include <Bridge.h>
#include <BridgeClient.h>


// MQTT_MAX_PACKET_SIZE : Maximum packet size
#define MQTT_MAX_PACKET_SIZE 512

// MQTT_KEEPALIVE : keepAlive interval in Seconds
#define MQTT_KEEPALIVE 15

#define MQTTPROTOCOLVERSION 3
#define MQTTCONNECT     1 << 4  // Client request to connect to Server
#define MQTTCONNACK     2 << 4  // Connect Acknowledgment
#define MQTTPUBLISH     3 << 4  // Publish message
#define MQTTPUBACK      4 << 4  // Publish Acknowledgment
#define MQTTPUBREC      5 << 4  // Publish Received (assured delivery part 1)
#define MQTTPUBREL      6 << 4  // Publish Release (assured delivery part 2)
#define MQTTPUBCOMP     7 << 4  // Publish Complete (assured delivery part 3)
#define MQTTSUBSCRIBE   8 << 4  // Client Subscribe request
#define MQTTSUBACK      9 << 4  // Subscribe Acknowledgment
#define MQTTUNSUBSCRIBE 10 << 4 // Client Unsubscribe request
#define MQTTUNSUBACK    11 << 4 // Unsubscribe Acknowledgment
#define MQTTPINGREQ     12 << 4 // PING Request
#define MQTTPINGRESP    13 << 4 // PING Response
#define MQTTDISCONNECT  14 << 4 // Client is Disconnecting
#define MQTTReserved    15 << 4 // Reserved

#define MQTTQOS0        (0 << 1)
#define MQTTQOS1        (1 << 1)
#define MQTTQOS2        (2 << 1)

#define MQTT_REPLY		"reply"
#define MQTT_NOTIFY 	"notify"

#define WIFI_SHIELD_WINDOW_SIZE	70

class dwOpenClient {
private:
   BridgeClient *_client; 
   uint8_t buffer[MQTT_MAX_PACKET_SIZE];   
   uint16_t nextMsgId;
   unsigned long lastOutActivity;
   unsigned long lastInActivity;
   bool pingOutstanding;
   void (*callback)(char*,uint8_t*,unsigned int);
   uint16_t readPacket(uint16_t*);
   uint8_t readByte();
   boolean write(uint8_t header, uint8_t* buf, uint16_t length);
   uint16_t writeString(char* string, uint8_t* buf, uint16_t pos);
   uint8_t *ip;
   char* domain;
   uint16_t port;
   int msgId;
   int rc;
   char szBuffer[256];
public:
	void doIt(void);
   	dwOpenClient();
   	dwOpenClient(uint8_t *, uint16_t, void(*)(char*,uint8_t*,unsigned int),BridgeClient *client);
   	dwOpenClient(char*, uint16_t, void(*)(char*,uint8_t*,unsigned int),BridgeClient *client);
   	boolean connect(char *);
   	boolean connect(char *, char *, char *);
   	boolean connect(char *, char *, uint8_t, uint8_t, char *);
   	boolean connect(char *, char *, char *, char *, uint8_t, uint8_t, char*);
   	void disconnect();
   	boolean publish(char *, const char *);
   	boolean publish(char *, uint8_t *, unsigned int);
   	boolean publish(char *, uint8_t *, unsigned int, boolean);
   	boolean publish_P(char *, const __FlashStringHelper *);
   	boolean subscribe(char *);
   	boolean unsubscribe(char *);
   	boolean loop();
   	boolean connected();
	void dwSleep(int);

	void jsonCreateBindThing( const char *thingKey, char *jsonText );
	void jsonCreateUnbindThing( const char *thingKey, char *jsonText );
	void jsonCreateLogPublish( const char *thingKey, const char *msgText, char *jsonText );
	void jsonCreatePropertyPublish( const char *thingKey, const char *propKey, const char *propValue, char *jsonText );
	void jsonCreatePropertyCurrent( char *thingKey, char *propKey, char *jsonText );
	void jsonCreateAlarmPublish( const char *thingKey, const char *alarmKey, int alarmState, const char *alarmMsg, char *jsonText );
	void jsonCreateLocationPublish( const char *thingKey, const char *locLatitude, const char *locLongitude, const char *locAltitude, const char *locSpeed, const char *locHeading, char *jsonText );
	void jsonCreateSetAttribute( const char *thingKey, const char *attribKey, const char *attribValue, char *jsonText );
	void jsonCreateGetAttribute( const char *thingKey, const char *attribKey, char *jsonText );

	int dwLogPublish( const char *thingKey, const char *szLogMsg );
	int dwLocationPublish( const char *thingKey, const char *szLat, const char *szLong, const char *szAlt, const char *szSpeed, const char *szHeading );
    int dwPropertyPublish( const char *thingKey, const char *szKey, const char *szValue );
    int dwAttributePublish( const char *thingKey, const char *szKey, const char *szValue );	
    int dwAlarmPublish( const char *thingKey, const char *szKey, int iState, const char *szMessage );
};

#endif
